package com.simuwang.daq.components;

import cn.hutool.core.collection.ListUtil;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.alibaba.excel.read.metadata.holder.ReadSheetHolder;
import com.simuwang.base.pojo.dto.report.CustomExcelTable;
import com.simuwang.base.pojo.dto.report.SimpleTable;

import java.io.Closeable;
import java.io.IOException;
import java.util.LinkedHashMap;
import java.util.List;

/**
 * @author wangzaijun
 * @date 2024/10/12 9:17
 * @description 自定义的excel多sheet解析事件监听器
 */
public class CustomExcelMultiSheetListener extends AnalysisEventListener<LinkedHashMap<Integer, Object>> implements Closeable {
    private final List<SimpleTable> tables = ListUtil.list(false);
    private SimpleTable table;

    private CustomExcelTable customExcelTable;

    public List<SimpleTable> getTables() {
        return tables;
    }

    @Override
    public void invoke(LinkedHashMap<Integer, Object> row, AnalysisContext analysisContext) {
        ReadSheetHolder sheetHolder = analysisContext.readSheetHolder();
        String sheetName = sheetHolder.getSheetName();
        if (sheetName.contains("封面") && sheetHolder.getSheetNo() == 0) {
            return;
        }
        @SuppressWarnings("unchecked")
        List<CustomExcelTable> customExcelTables = (List<CustomExcelTable>) analysisContext.getCustom();
        String title = ReportParseUtils.cleaningValue(row.get(1));
        if (title != null) {
            for (CustomExcelTable customExcelTable : customExcelTables) {
                String tableTitle = customExcelTable.title();
                if (title.equals(tableTitle) || title.contains(tableTitle)) {
                    this.table = new SimpleTable(tableTitle, customExcelTable.colCount());
                    this.customExcelTable = customExcelTable;
                    this.tables.add(this.table);
                    return;
                }
            }
        }
        int rowCount = this.customExcelTable.rowCount() <= 0 ? Integer.MAX_VALUE : this.customExcelTable.rowCount();
        if (this.table != null && this.table.getRowCount() < rowCount) {
            List<String> tableRow = ListUtil.list(true);
            int colCount = this.table.getColCount() <= 0 ? row.size() : this.table.getColCount() + this.customExcelTable.startCol();
            for (int i = this.customExcelTable.startCol(); i < colCount; i++) {
                tableRow.add(ReportParseUtils.cleaningValue(row.get(i)));
            }
            this.table.addRow(tableRow);
        }
    }

    @Override
    public void doAfterAllAnalysed(AnalysisContext analysisContext) {

    }

    @Override
    public void close() throws IOException {
        this.tables.clear();
    }
}
