package com.simuwang.daq.components.report.parser.excel;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.read.builder.ExcelReaderBuilder;
import com.simuwang.base.common.exception.ReportParseException;
import com.simuwang.base.mapper.EmailFieldMappingMapper;
import com.simuwang.base.pojo.dto.report.CustomExcelTable;
import com.simuwang.base.pojo.dto.report.ReportData;
import com.simuwang.base.pojo.dto.report.ReportParserParams;
import com.simuwang.base.pojo.dto.report.SimpleTable;
import com.simuwang.daq.components.CustomExcelMultiSheetListener;
import com.simuwang.daq.components.report.parser.AbstractReportParser;

import java.io.IOException;
import java.util.List;

public abstract class AbstractExcelReportParser<T extends ReportData> extends AbstractReportParser<T> {
    public AbstractExcelReportParser(EmailFieldMappingMapper fieldMappingMapper) {
        super(fieldMappingMapper);
    }

    @Override
    public T parse(ReportParserParams params) throws IOException, ReportParseException {
        this.init();
        // 配置excel解析
        CustomExcelMultiSheetListener excelListener = new CustomExcelMultiSheetListener();
        ExcelReaderBuilder readerBuilder = EasyExcel.read(params.getFilepath());
        readerBuilder.sheet();
        readerBuilder.customObject(this.customExcelTables());
        readerBuilder.registerReadListener(excelListener);
        readerBuilder.doReadAll();
        // 解析出表格
        List<SimpleTable> tables = excelListener.getTables();
        return this.excelParse(tables, params);
    }

    @Override
    protected void cleaningReportData(T reportData) {
        // cleaning.
    }

    protected abstract List<CustomExcelTable> customExcelTables();

    protected abstract T excelParse(List<SimpleTable> tables, ReportParserParams params);
}
